Imports System.Reflection
Imports System.Data
Imports System.Collections.Generic
Imports System.ComponentModel

''' <summary>enumeriert die Rows eines DataViews oder einer BindingSource</summary>
Public Class RowEnumerable(Of T As DataRow) : Inherits Enumerable(Of T)

   Private ReadOnly _Owner As DataView(Of T)

   Public Sub New(ByVal Owner As DataView(Of T))
      _Owner = Owner
   End Sub

   Private _IndexEnumerator As IEnumerator(Of Integer)
   Private _RowCache As T()

   Private Shared GetEnumeratorInfo As MethodInfo
   Private Shared IndexInfo As FieldInfo
   Private Shared RecordManagerInfo As FieldInfo
   Private Shared RowCacheInfo As FieldInfo

   Shared Sub New()
      Dim BindFlags As BindingFlags = _
         BindingFlags.Instance Or BindingFlags.Public Or BindingFlags.NonPublic
      IndexInfo = GetType(Data.DataView).GetField("index", BindFlags)
      GetEnumeratorInfo = IndexInfo.FieldType.GetMethod("GetEnumerator", BindFlags)
      RecordManagerInfo = GetType(DataTable).GetField("recordManager", BindFlags)
      RowCacheInfo = RecordManagerInfo.FieldType.GetField("rows", BindFlags)
   End Sub

   Protected Overrides Sub Prepare()
      'get an enumerator from Dataview-Index
      'get the DataRowCache from Dataview.Table
      Dim Index As Object = IndexInfo.GetValue(_Owner)
      _IndexEnumerator = DirectCast( _
         GetEnumeratorInfo.Invoke(Index, New Object() {0}),  _
         IEnumerator(Of Integer))
      Dim RecordManager As Object = RecordManagerInfo.GetValue(_Owner.Table)
      _RowCache = DirectCast(RowCacheInfo.GetValue(RecordManager), T())
      MyBase.Prepare()
   End Sub

   Protected Overrides Function TryMovenext(ByRef Current As T) As Boolean
      TryMovenext = _IndexEnumerator.MoveNext
      If TryMovenext Then
         Current = _RowCache(_IndexEnumerator.Current)
      End If
   End Function

   Protected Overrides Sub CleanUp()
      MyBase.CleanUp()
   End Sub

End Class
