﻿Imports System.IO
Imports System.Text.RegularExpressions
Imports System.Xml
Imports System.Text

Partial Public Class dtsSolutionExplorer

   Private _Solution As PathEx                               'Pfad-Informationen des Solution-Files
   Private _CommonPathLength As Integer
   Private _Status As StatusLogger
   Private _ViewMode As ViewModeEnum
   Private _FileTypeNameFinder As DataView(Of FileTypeRow)
   Private _FileTypeIDFinder As DataView(Of FileTypeRow)

   Private Sub dtsSolutionExplorer_Initialized(ByVal sender As Object, ByVal e As EventArgs) _
         Handles Me.Initialized
      _FileTypeIDFinder = DataView(Of FileTypeRow).FromTable(Me.FileType) _
         .Sort(Me.FileType.FileTypeIDColumn.ColumnName).EndInit
      _FileTypeNameFinder = DataView(Of FileTypeRow).FromTable(Me.FileType) _
         .Sort(Me.FileType.TypeNameColumn.ColumnName).EndInit
   End Sub

   Friend ReadOnly Property CommonPathLength() As Integer
      Get
         Return _CommonPathLength
      End Get
   End Property

   Friend Function GetFileTypIndex(ByVal rwFile As FilesRow) As Integer
      Return _FileTypeIDFinder.Find(rwFile.FileTypeID)
   End Function

   Friend Sub FillTables( _
         ByVal Solution As PathEx, _
         ByVal Status As StatusLogger, _
         ByVal ViewMode As ViewModeEnum)
      _Status = Status
      _Solution = Solution
      _ViewMode = ViewMode
      _Status.Log("Lade Solution")
      'DataTables mit Informationen aus Solutionfile und ProjectFiles füllen
      Dim SolutionFileTypRow, ProjFileTypRow As dtsSolutionExplorer.FileTypeRow
      With Me
         .Project.Clear()
         .Files.Clear()
         .FileType.Clear()
         SolutionFileTypRow = .FileType.AddFileTypeRow("SolutionFile")
         ProjFileTypRow = .FileType.AddFileTypeRow("ProjectFile")
      End With

      Dim SolutionFileContent = File.ReadAllText(_Solution.Fullname)
      Me.Files.AddFilesRow(Nothing, Nothing, SolutionFileTypRow, _Solution.Fullname)
      Dim Pattern = String.Concat( _
          "Project\(""\{.+\}""\) = "".+"", ""(?<ProjRelPath>.+)"", ""(?<ProjGuid>\{.+\})""")
      'Der Pattern matcht einen kompletten Projekteintrag
      '!                             im Solutionfile. Aufzeichnende Gruppen: 1) <ProjRelPath>:
      '!                                   Pfad zum Proj-File, relativ zum Folder der Solution
      '!                                                     2) <ProjGuid>: Guid des Projektes
      Dim Mts = Regex.Matches(SolutionFileContent, Pattern)
      If Mts.Count = 0 Then Throw New ArgumentOutOfRangeException( _
         "_Solution.Fullname", _Solution.Fullname, "Kein Projekt gefunden in")
      For Each Mt As Match In Mts
         Dim ProjRelPath = Mt.Groups("ProjRelPath").Value
         If ProjRelPath.StartsWith("..\") AndAlso _
                 _ViewMode = ViewModeEnum.Exclude_Extern_Projects Then
            'Externes Projekt ausschließen
         Else
            Dim ProjFullname = _Solution.AbsolutePath(ProjRelPath)
            If Not File.Exists(ProjFullname) Then
               _Status.Log("""", ProjFullname, """ nicht gefunden!")
               Continue For
            End If
            'ProjectFile in die Project-Table einspeichern
            Dim rwProj = Me.Project.AddProjectRow(ProjRelPath, Mt.Groups("ProjGuid").Value)
            CollectFiles(rwProj)                     'alle ProjectItem-Files in Files-Table sammeln
            Me.Files.AddFilesRow(Path.GetFileName(ProjFullname), rwProj, ProjFileTypRow, ProjFullname)
         End If
      Next
      InitCommonPathLength()
   End Sub

   Private _rgxPlatform As New Regex("(</PropertyGroup\>)|(AnyCPU|x86|x64|Itanium)|(<PropertyGroup)")
   Public Sub SetTargetPlatform(ByVal key As String)
      Dim isInPropertyGroup = 0
      Dim hasChanges = False
      Dim check As MatchEvaluator = Function(mt)
                                       Dim add = mt.Groups.Cast(Of Group).Skip(1).IndexOf(Function(gp) gp.Success) - 1
                                       If add <> 0 Then
                                          isInPropertyGroup += add
                                       ElseIf isInPropertyGroup > 0 Then
                                          If key <> mt.Value Then hasChanges = True
                                          Return key
                                       End If
                                       Return mt.Value
                                    End Function
      Dim msgTitle = "changing to " & key
      For Each rwProj In Me.Project
         hasChanges = False
         Dim projPath As New PathEx(_Solution.AbsolutePath(rwProj.RelPath))
         Dim txt = _rgxPlatform.Replace(File.ReadAllText(projPath.Fullname), check)
         If isInPropertyGroup <> 0 Then
            MessageBox.Show("inkonsistentes Xml in " & projPath.FileName, msgTitle)
         Else
            MessageBox.Show("Rewrite ".And(projPath.FileName, ": ", hasChanges), msgTitle)
            If hasChanges Then File.WriteAllText(projPath.Fullname, txt)
         End If
      Next
      With File.ReadAllText(_Solution.Fullname)
         MessageBox.Show("Rewrite ".And(_Solution.FileName, ": True"), msgTitle)
         Dim i = .IndexOf("Global")
         If i < 0 Then Return
         File.WriteAllText(_Solution.Fullname, .Remove(i))
      End With
   End Sub

   Public Sub ChangeProjectVersion(version As VersionInfo)
      For Each rwProj In Me.Project
         Dim ProjPathEx As New PathEx(_Solution.AbsolutePath(rwProj.RelPath))
         Dim XDoc As New XmlDocument
         XDoc.Load(ProjPathEx.Fullname)
         Dim NSMngr As New XmlNamespaceManager(XDoc.NameTable)
         NSMngr.AddNamespace("def", XDoc.DocumentElement.NamespaceURI)
         With XDoc.DocumentElement.Attributes("ToolsVersion")
            If .notnull Then .Value = version.ToolsVersion
         End With
         Dim Xel = XDoc.DocumentElement("PropertyGroup")
         Dim cld = Xel("FileUpgradeFlags")
         If cld IsNot Nothing Then
            Xel.RemoveChild(cld)
            Xel.RemoveChild(Xel("UpgradeBackupLocation"))
            Xel.RemoveChild(Xel("OldToolsVersion"))
         End If
         cld = Xel("TargetFrameworkVersion")
         If cld Is Nothing Then
            If version.TargetFrameworkVersion = "" Then Return
            cld = XDoc.CreateElement("TargetFrameworkVersion")
            cld.InnerText = version.TargetFrameworkVersion
            Xel.AppendChild(cld)
         Else
            If version.TargetFrameworkVersion = "" Then
               Xel.RemoveChild(cld)
            Else
               cld.InnerText = version.TargetFrameworkVersion
            End If
         End If
         XDoc.Save(ProjPathEx.Fullname)
         ChangeAppConfig(rwProj)
      Next
   End Sub

   Private Sub ChangeAppConfig(rwProj As ProjectRow)
      Dim rwConfig = rwProj.GetFilesRows.FirstOrDefault(Function(r) r.Fullname.EndsWith("App.Config", StringComparison.OrdinalIgnoreCase))
      If rwConfig Is Nothing Then Return
      Dim XDoc As New XmlDocument
      XDoc.Load(rwConfig.Fullname)
      Dim Xel = XDoc.DocumentElement.SelectSingleNode("startup/supportedRuntime")
      If Xel Is Nothing Then Return
      Xel.ParentNode.RemoveChild(Xel)
      XDoc.Save(rwConfig.Fullname)
   End Sub

   Private Sub CollectFiles(ByVal rwProj As dtsSolutionExplorer.ProjectRow)
      'abhängige DataTable 'Files' mit von rwProj abhängigen Datensätzen füllen
      Dim ProjPathEx As New PathEx(_Solution.AbsolutePath(rwProj.RelPath))
      Dim XDoc As New XmlDocument
      XDoc.Load(ProjPathEx.Fullname)
      Dim NSMngr As New XmlNamespaceManager(XDoc.NameTable)
      NSMngr.AddNamespace("def", XDoc.DocumentElement.NamespaceURI)
      For Each Xattr As XmlAttribute In XDoc.DocumentElement.SelectNodes( _
            "def:ItemGroup//@Include", NSMngr)
         Dim ItemType As String = Xattr.OwnerElement.Name
         'Xattr.OwnerElement.Name (falls unbekannt) in die FileType-Table
         '! einspeichern
         Dim rwFileTyp = If( _
            _FileTypeNameFinder.FindRows(ItemType).FirstOrDefault, _
            Me.FileType.AddFileTypeRow(ItemType))
         Dim sFile = UnEscape(Xattr)
         Dim FileFullname = ProjPathEx.AbsolutePath(sFile)
         If IsToCollect(ItemType, FileFullname) Then
            If File.Exists(FileFullname) OrElse Directory.Exists(FileFullname) Then
               Me.Files.AddFilesRow(sFile, rwProj, rwFileTyp, FileFullname)
            Else
               _Status.Log("""", FileFullname, """ nicht gefunden!")
            End If
         End If
      Next
   End Sub

   Private _rgxUnEscape As New Regex("\%\d\d")
   Private Function UnEscape(xnd As XmlNode) As String
      Return _rgxUnEscape.Replace(xnd.Value, AddressOf UnEscapeEvaluator)
   End Function
   Private Function UnEscapeEvaluator(mt As Match) As String
      Dim by = Byte.Parse(mt.Value.Substring(1), Globalization.NumberStyles.AllowHexSpecifier)
      Return Encoding.ASCII.GetChars({by})
   End Function

   Private Function IsToCollect(ByVal ItemType As String, ByVal FileFullname As String) As Boolean
      Select Case ItemType
         Case "Reference", "ProjectReference", "Import", "BootstrapperPackage"
            'OwnerElemente dieses Namens bezeichnen mit dem @Include - Attribut keine zu
            '! berücksichtigende Datei (doch, "ProjectReference" schon, aber diese wird ja
            '! schon im Solution-File ausgewertet)

         Case Else
            'Include-Attribut aller anderen OwnerElemente in die File-Table speichern
            Select Case _ViewMode
               Case ViewModeEnum.UserCode_Only
                  Select Case Path.GetExtension(FileFullname).ToLower
                     Case ".vb", ".cs"
                        If File.Exists(FileFullname) Then
                           Static rgxGenerated As New Regex( _
                               "(' <auto-generated>)|(<\S*DesignerGenerated\(\)>)", _
                               RegexOptions.Compiled)
                           Return Not rgxGenerated.IsMatch(FileReadWriter.Read(FileFullname, 300))
                        Else
                           _Status.Log("""", FileFullname, """ nicht gefunden!")
                        End If
                  End Select

               Case ViewModeEnum.Exclude_Extern_Projects
                  Return FileFullname.StartsWith(_Solution.DirectoryName)

               Case ViewModeEnum.Complete
                  Return True

               Case Else
                  Throw New Exception("Unknown ViewModeEnum-Value: ".And(_ViewMode))
            End Select

      End Select
      Return False
   End Function

   Private Sub InitCommonPathLength()
      'initiiere CommonPathLength, die Länge des Pfades des alle Files enthaltenden Ordners
      Dim Fullnames As New List(Of String)
      Me.Files.Select(Function(rwFile) rwFile.Fullname).ForEach(AddressOf Fullnames.Add)
      _CommonPathLength = PathEx.GetCommonPathLength(Fullnames)
      If _CommonPathLength < 0 Then
         'mögl. Fehler: 1) keinen FileFullname, 2) nur einen FileFullname, 3) FileFullnames
         '! auf verschiedenen Laufwerken
         Throw New Exception("Fehler mit der Liste der FileFullnames")
      End If
   End Sub

End Class
